<?php

namespace Drupal\opigno_module\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\h5p\H5PDrupal\H5PDrupal;
use Drupal\opigno_module\Entity\OpignoActivity;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\SettingsCommand;

/**
 * Class ActivitiesBrowserController.
 *
 * @package Drupal\opigno_module\Controller
 */
class ExternalPackageController extends ControllerBase {

  public static function ajaxFormExternalPackageCallback(&$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();

    // If errors, returns the form with errors and messages.
    if ($form_state->hasAnyErrors()) {
      return $form;
    }

    if ($entity = $form_state->getUserInput()['activity']) {
      $item = [];
      $item['id'] = $entity->id();
      $item['name'] = $entity->getName();

      $command = new SettingsCommand([
        'formValues' => $item,
        'messages' => drupal_get_messages(null, true),
      ], TRUE);
    } else {
      $command = new SettingsCommand([
        'messages' => drupal_get_messages(null, true),
      ], TRUE);
    }

    $response->addCommand($command);

    return $response;
  }

  /**
   * Submit handler added in the form
   * via the function opigno_module_form_alter().
   */
  public static function ajaxFormExternalPackageFormSubmit($form, FormState &$form_state) {
    $fid = $form_state->get('package');
    $file = File::load($fid);
    $params = \Drupal::routeMatch()->getParameters();
    $module = $params->get('opigno_module');

    // If one information missing, return an error.
    if (!isset($module)) {
      // TODO: Add an error message here.
      return;
    }

    if (!empty($file)) {
      // Get file extension.
      $extension = ExternalPackageController::getFileExtension($file->getFilename());
      switch ($extension) {
        case 'zip':
          // Check file type. Can be "scorm" or "tincan".
          $type = ExternalPackageController::checkPackageType($file);

          if (!$type) {
            \Drupal::messenger()->addError(t("Package does not contain required files."));
            return $form_state->setRebuild();
          }
          // Create activity.
          $entity = ExternalPackageController::createActivityByPackageType($file, $type, $form, $form_state);
          break;

        case 'h5p':
          $entity = ExternalPackageController::createActivityByPackageType($file, 'h5p', $form, $form_state);
          break;
      }
      if (!$entity) {
        \Drupal::messenger()->addWarning(t("Can't create activity."));
      } else {
        // Clear user input.
        $input = $form_state->getUserInput();
        // We should not clear the system items from the user input.
        $clean_keys = $form_state->getCleanValueKeys();
        $clean_keys[] = 'ajax_page_state';

        foreach ($input as $key => $item) {
          if (!in_array($key, $clean_keys)
            && substr($key, 0, 1) !== '_') {
            unset($input[$key]);
          }
        }

        // Store new entity for display in the AJAX callback.
        $input['activity'] = $entity;
        $form_state->setUserInput($input);

        // Rebuild the form state values.
        $form_state->setRebuild();
        $form_state->setStorage([]);

        // Assign activity to module if entity is new.
        if (!isset($item_id)) {
          /** @var \Drupal\opigno_module\Controller\OpignoModuleController $opigno_module_controller */
          $opigno_module_controller = \Drupal::service('opigno_module.opigno_module');
          $opigno_module_controller->activitiesToModule([$entity], $module);
        }
      }
    }

    return $form_state->setRebuildInfo([t("Can't create an activity. File can't be uploaded.")]);
  }

  /**
   * Function for checking what package type was downloaded.
   *
   * @return string | boolen
   *   Returned 'scorm' or 'tincan' or FALSE.
   *
   */
  protected function checkPackageType($file) {
    // Unzip file.
    $path = \Drupal::service('file_system')->realpath($file->getFileUri());
    $zip = new \ZipArchive();
    $result = $zip->open($path);
    if ($result === TRUE) {
      $extract_dir = 'public://external_package_extracted/package_' . $file->id();
      $zip->extractTo($extract_dir);
      $zip->close();

      // This is a standard: these files must always be here.
      $scorm_file = $extract_dir . '/imsmanifest.xml';
      $tincan_file = $extract_dir . '/tincan.xml';

      if (file_exists($scorm_file)) {
        $package_type = 'scorm';
      }
      elseif (file_exists($tincan_file)) {
        $package_type = 'tincan';
      }
      // Delete extracted archive.
      file_unmanaged_delete_recursive($extract_dir);

      if (isset($package_type)) {
        return $package_type;
      }

    }
    return FALSE;
  }

  /**
   * Function for creating activity depending file type.
   *
   * @param \Drupal\File\entity\File $file
   * @param string $package_type
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return object | boolean
   *   Returned activity object or FALSE
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function createActivityByPackageType(File $file, $package_type, array &$form, FormStateInterface $form_state) {

    switch ($package_type) {
      // Create Scorm_package activity.
      case 'scorm':
        $activity = OpignoActivity::create([
          'type' => 'opigno_scorm',
          'name' => $form_state->getValue('name'),
          'opigno_scorm_package' => [
            'target_id' => $file->id(),
          ],
        ]);

        $activity->save();
        break;

      // Create Tincan_package activity.
      case 'tincan':
        $has_library = opigno_tincan_api_tincanphp_is_installed();
        if (!$has_library) {
          \Drupal::messenger()->addError(t('Impossible to create a new TinCan Package question type.'));
          return FALSE;
        };
        $activity = OpignoActivity::create([
          'type' => 'opigno_tincan',
          'name' => $form_state->getValue('name'),
          'opigno_tincan_package' => [
            'target_id' => $file->id(),
          ],
        ]);

        $activity->save();
        break;

      // Create Interactive content activity.
      case 'h5p':
        $h5p_content_id = ExternalPackageController::createH5pContent($file);
        if (!$h5p_content_id) {
          \Drupal::messenger()->addError(t("Can't create h5p content. Wrong h5p package."));
          return FALSE;
        };
        $activity = OpignoActivity::create([
          'type' => 'opigno_h5p',
          'name' => $form_state->getValue('name'),
          'opigno_h5p' => [
            'h5p_content_id' => $h5p_content_id,
          ],
        ]);

        $activity->save();
        break;
    }

    return $activity;
  }

  /**
   * Function for creating h5p content for Interactive content activity.
   *
   * @param \Drupal\file\Entity\File $file
   *
   * @return int | boolean
   *   Return h5p content id or FALSE.
   */
  protected function createH5pContent(File $file) {
    $file_field = 'package';

    // Prepare temp folder.
    $interface = H5PDrupal::getInstance('interface', $file_field);
    $temporary_file_path = 'public://external_packages';

    // Tell H5P Core where to look for the files.
    $interface->getUploadedH5pPath(\Drupal::service('file_system')
      ->realpath($file->getFileUri()));
    $interface->getUploadedH5pFolderPath(\Drupal::service('file_system')
      ->realpath($temporary_file_path));

    // Call upon H5P Core to validate the contents of the package.
    $validator = H5PDrupal::getInstance('validator', $file_field);
    $validator->isValidPackage();

    // Store the uploaded file.
    $storage = H5PDrupal::getInstance('storage', $file_field);

    $content = [
      'id' => NULL,
      'uploaded' => TRUE,
      'disable' => 0,
    ];

    // Save and update content id.
    $storage->savePackage($content);
    $content_id = $storage->contentId;

    if (!$content_id) {
      return FALSE;
    };
    return $content_id;
  }

  /**
   * @param \Drupal\opigno_module\Form\string $file_name
   *
   * @return string
   */
  protected function getFileExtension($file_name) {
    return substr(strrchr($file_name, '.'), 1);
  }
}
